# -*- coding: utf-8 -*-

ORDER_ACCEPTED_MESSAGE = "Order Accepted"
ORDER_FILLED_MESSAGE = "Order Filled"
ORDER_PARTIALLY_FILLED_MESSAGE = "Order Partially filled"

# App Universe kstable Column name
DATETIME_COLUMN = "datetime"
SID_COLUMN = "sid"
PRICE_COLUMN = "price"

# Special column for singla data
SIGNAL_DATA_SID_COLUMNS = "__keystone_signal_data_column__"